from config.dashgo_conf import SqlDbConf
from playhouse.pool import PooledMySQLDatabase
from peewee import SqliteDatabase
from server import server
from playhouse.shortcuts import ReconnectMixin


if SqlDbConf.RDB_TYPE == 'mysql':
    # 断线重连+连接池
    class ReconnectPooledMySQLDatabase(ReconnectMixin, PooledMySQLDatabase):
        _instance = None

        @classmethod
        def get_db_instance(cls):
            if not cls._instance:
                cls._instance = cls(
                    database=SqlDbConf.DATABASE,
                    max_connections=SqlDbConf.POOL_SIZE,
                    user=SqlDbConf.USER,
                    password=SqlDbConf.PASSWORD,
                    host=SqlDbConf.HOST,
                    port=SqlDbConf.PORT,
                    stale_timeout=300,
                )
            return cls._instance
elif SqlDbConf.RDB_TYPE == 'sqlite':
    sqlite_db = SqliteDatabase(SqlDbConf.SQLITE_DB_PATH)
else:
    raise NotImplementedError('Unsupported database type')


def db():
    if SqlDbConf.RDB_TYPE == 'mysql':
        return ReconnectPooledMySQLDatabase.get_db_instance()
    elif SqlDbConf.RDB_TYPE == 'sqlite':
        return sqlite_db
    else:
        raise NotImplementedError('Unsupported database type')


# 判断是否存在SysUser表，如不存在则初始化库
def initialize_database():
    db_instance = db()
    if not db_instance.table_exists('sys_user'):
        from .entity.table_user import SysUser, SysRoleAccessMeta, SysUserRole, SysGroupUser, SysRole, SysGroupRole, SysGroup
        from datetime import datetime
        import hashlib

        db_instance.create_tables(
            [
                SysUser,
                SysRoleAccessMeta,
                SysUserRole,
                SysGroupUser,
                SysRole,
                SysGroupRole,
                SysGroup,
            ]
        )
        SysRole.create(
            role_name='admin',
            role_status=True,
            update_datetime=datetime.now(),
            update_by='admin',
            create_datetime=datetime.now(),
            create_by='admin',
            role_remark='超级管理员角色',
        )
        SysUser.create(
            user_name='admin',
            user_full_name='超级管理员',
            password_sha256=hashlib.sha256('admin123'.encode('utf-8')).hexdigest(),
            user_status=True,
            user_sex='未知',
            user_email='',
            phone_number='',
            create_by='admin',
            create_datetime=datetime.now(),
            update_by='admin',
            update_datetime=datetime.now(),
            user_remark='',
            otp_secret='',
        )
        SysUserRole.create(user_name='admin', role_name='admin')


initialize_database()


# 自动管理数据库上下文
@server.before_request
def _db_connect():
    db().connect(reuse_if_open=True)


@server.teardown_request
def _db_close(exc):
    _db = db()
    if not _db.is_closed():
        _db.close()
